==> d3d-0.1.1.tgz: package/lib/pie.js <==
import * as d3 from 'd3'
import * as d3dPrism from './prism'
import {PI, select} from './utils'

let count = 0

const COLORS = [
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
]

export function draw(parent, data, {rx, ry, h, ir}) {
  _draw(parent, data, {rx, ry, h, ir})
  return
  select(parent).transition().duration(500).attrTween('fill', function (d) {
    this._option = this._option || data
    const interpolator = d3.interpolate(this._option, data)
    return t => {
      const opt = interpolator(t)
      _draw(parent, opt, {rx, ry, h, ir})
    }
  })
}

export function _draw(parent, data, {rx, ry, h, ir}) {
  const pie = select(parent)
  const pieEl = pie._groups[0][0]
  pieEl._option = data // {data, rx, ry, h, ir}

  const arcClass = 'd3d-arc-'

  // sort: 对data进行map得到option再跟据远近排序，调整显示层次
  const arcOptions = d3.pie()
    // .sort((a, b) => 1)
    .value(d => d)(data)
    .map((opt, i) => {
      const interpolator = d3.interpolate(opt.startAngle, opt.endAngle)
      const diffAngle270 = diffAngle(1.5 * PI)
      opt.angleDiffs = {start: diffAngle270(interpolator(.001)), end: diffAngle270(interpolator(.999))}
      return Object.assign(opt, {rx, ry, h, ir, color: COLORS[i], i})
    })
    .sort((a, b) => {
      if (a.endAngle <= b.startAngle) {
        return a.angleDiffs.end - b.angleDiffs.start
      } else {
        return a.angleDiffs.start - b.angleDiffs.end
      }
    })

  // update
  const update = pie.selectAll('.' + arcClass).data(arcOptions)
    .each(function (arcOpt) {
      // 调整数据后，显示层次会变化，所以只能find对应的arc，再进行更新
      const arc = [...pieEl.children].find(arc => arc.index === arcOpt.index)
      d3dPrism._update(arc, arcOpt, arcOpt.startAngle, arcOpt.endAngle, rx, ry, h, ir, arc._option.color)
      pieEl.appendChild(arc) // 把find到的arc, append到最后，调整层次
      arc._option = arcOpt
      select(arc).attr('data', arcOpt.data)
    })

  // enter
  update.enter().append('g')
    .attr('class', (d, i) => arcClass + ' ' + d.index)
    .each(function (arcOpt, i) {
      const color = arcOpt.color
      const arc = d3dPrism.update(this, arcOpt, arcOpt.startAngle, arcOpt.endAngle, rx, ry, h, ir, color)
        .on('mouseover', function () {
          d3dPrism._update(this, {...this._option, color: d3.hsl(color).darker(-.5)})
        })
        .on('mouseout', function () {
          d3dPrism._update(this, {...this._option, color})
        })
        .on('click', function () {
          pieEl.curArc && (pieEl.curArc !== this) && d3dPrism.update(pieEl.curArc, {...pieEl.curArc._option, rx, ry, h, ir})
          d3dPrism.update(this, {
            ...this._option,
            h: h * 2.5 - this._option.h,
            // rx: rx * 2.5 - this._option.rx,
            // ry: ry * 2.5 - this._option.ry,
            ir: ir * 1.9 - this._option.ir,
          })
          pieEl.curArc = this
        })
      const arcEl = arc._groups[0][0]
      arcEl.index = arcOpt.index
    })

  return pie
}

function diffAngle(anchor) {
  return function (angle) {
    const absDiff = Math.abs(angle - anchor)
    return absDiff < PI
      ? absDiff
      : 2 * PI - absDiff
  }
}

==> d3d-0.1.6.tgz: package/lib/pie.js <==
import * as d3 from 'd3'
import * as d3dPrism from './prism'
import {PI, select} from './utils'

let count = 0

const COLORS = [
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
  '#4eebab',
  '#4CCCFE',
  '#af84d6',
  '#ffad48',
  '#df786d',
]

export function draw(parent, data, {rx, ry, h, ir, colors}, onItemClick) {
  _draw(parent, data, {rx, ry, h, ir, colors}, onItemClick)
  return
  select(parent).transition().duration(500).attrTween('fill', function (d) {
    this._option = this._option || data
    const interpolator = d3.interpolate(this._option, data)
    return t => {
      const opt = interpolator(t)
      _draw(parent, opt, {rx, ry, h, ir})
    }
  })
}

export function _draw(parent, data, {rx, ry, h, ir, colors = COLORS}, onItemClick) {
  const pie = select(parent)
  const pieEl = pie._groups[0][0]
  pieEl._option = data // {data, rx, ry, h, ir}

  const arcClass = 'd3d-arc-'

  // sort: 对data进行map得到option再跟据远近排序，调整显示层次
  const arcOptions = d3.pie()
    // .sort((a, b) => 1)
    .value(d => typeof (d) === 'number' ? d : d.value)(data)
    .map((opt, i) => {
      const interpolator = d3.interpolate(opt.startAngle, opt.endAngle)
      const diffAngle270 = diffAngle(1.5 * PI)
      opt.angleDiffs = {start: diffAngle270(interpolator(.001)), end: diffAngle270(interpolator(.999))}
      return Object.assign(opt, {rx, ry, h, ir, color: colors[i], i})
    })
    .sort((a, b) => {
      if (a.endAngle <= b.startAngle) {
        return a.angleDiffs.end - b.angleDiffs.start
      } else {
        return a.angleDiffs.start - b.angleDiffs.end
      }
    })

  // update
  const update = pie.selectAll('.' + arcClass).data(arcOptions)
    .each(function (arcOpt) {
      // 调整数据后，显示层次会变化，所以只能find对应的arc，再进行更新
      const arc = [...pieEl.children].find(arc => arc.index === arcOpt.index)
      d3dPrism._update(arc, arcOpt, arcOpt.startAngle, arcOpt.endAngle, rx, ry, h, ir, arc._option.color)
      pieEl.appendChild(arc) // 把find到的arc, append到最后，调整层次
      arc._option = arcOpt
      select(arc).attr('data', arcOpt.data)
    })

  // enter
  update.enter().append('g')
    .attr('class', (d, i) => arcClass + ' ' + d.index)
    .each(function (arcOpt, i) {
      const color = arcOpt.color
      const arc = d3dPrism.update(this, arcOpt, arcOpt.startAngle, arcOpt.endAngle, rx, ry, h, ir, color)
        .on('mouseover', function () {
          d3dPrism._update(this, {...this._option, color: d3.hsl(color).darker(-.5)})
        })
        .on('mouseout', function () {
          d3dPrism._update(this, {...this._option, color})
        })
        .on('click', function (ev) {
          ev.stopPropagation()

          pieEl.curArc && (pieEl.curArc !== this) && d3dPrism.update(pieEl.curArc, {...pieEl.curArc._option, rx, ry, h, ir})
          d3dPrism.update(this, {
            ...this._option,
            h: h * 3 - this._option.h,
            // rx: rx * 2.5 - this._option.rx,
            // ry: ry * 2.5 - this._option.ry,
          })

          if (pieEl.curArc === this && this._option.h !== h) {
            showText(pieEl, '')
          } else {
            onItemClick && onItemClick(this._option, this, pieEl)
          }

          pieEl.curArc = this
        })
      const arcEl = arc._groups[0][0]
      arcEl.index = arcOpt.index
    })

  // text
  !pieEl._text && (pieEl._text = select(pieEl).append('text')
      .style('fill', 'white')
      .style('text-anchor', 'middle')
      .style('font-size', '.8em')
      .style('pointer-events', 'none')
      .style('transition', 'opacity 1s')
      .style('opacity', 0)
      .attr('y', -ry * 1.8)
      .text('文字描述')
  )

  return pie
}

export function showText(pieEl, text) {
  pieEl._text.style('opacity', text ? 1 : 0)
  text && pieEl._text.text(text)
}

function diffAngle(anchor) {
  return function (angle) {
    const absDiff = Math.abs(angle - anchor)
    return absDiff < PI
      ? absDiff
      : 2 * PI - absDiff
  }
}
